@@ -1,14 +1,15 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
+import re
 import time
 
 from google.protobuf.json_format import MessageToJson
 
 from . import BaseDriver
-from ..types.querylang.queryset.dunderkey import dunder_get
 from ..excepts import UnknownControlCommand, RuntimeTerminated, NoExplicitMessage
 from ..proto import jina_pb2
+from ..types.querylang.queryset.dunderkey import dunder_get
 
 
 class BaseControlDriver(BaseDriver):
@@ -80,6 +81,22 @@ class ControlReqDriver(BaseControlDriver):
         elif self.req.command == 'STATUS':
             self.envelope.status.code = jina_pb2.StatusProto.READY
             self.req.args = vars(self.runtime.args)
+        elif self.req.command == 'IDLE':
+            pass
+        elif self.req.command == 'CANCEL':
+            pass
+        elif self.req.command == 'RELOAD':
+            if self.req.targets and self.runtime.__class__.__name__ == 'ZEDRuntime':
+                patterns = self.req.targets
+                if isinstance(patterns, str):
+                    patterns = [patterns]
+                for p in patterns:
+                    if re.match(p, self.runtime.name):
+                        self.logger.info(
+                            f'reloading the Executor `{self.runtime._executor.name}` in `{self.runtime.name}`'
+                        )
+                        self.runtime._load_executor()
+                        break
         else:
             raise UnknownControlCommand(f'don\'t know how to handle {self.req.command}')
 
@@ -143,6 +160,10 @@ class RouteDriver(ControlReqDriver):
             # all the time
             # (2) this driver is used in a ROUTER-DEALER fan-out setting,
             # where some dealer is broken/fails to start, so `idle_dealer_ids` is empty
+            # IDLE requests add the dealer id to the router. Therefore, it knows which dealer would be available for
+            # new data requests.
+            # CANCEL requests remove the dealer id from the router. Therefore, it can not send any more data requests
+            # to the dealer.
         elif self.req.command == 'IDLE':
             self.idle_dealer_ids.add(self.envelope.receiver_id)
             self.logger.debug(
@@ -151,10 +172,32 @@ class RouteDriver(ControlReqDriver):
             if self.is_polling_paused:
                 self.runtime._zmqlet.resume_pollin()
                 self.is_polling_paused = False
-            raise NoExplicitMessage
+        elif self.req.command == 'CANCEL':
+            if self.envelope.receiver_id in self.idle_dealer_ids:
+                self.idle_dealer_ids.remove(self.envelope.receiver_id)
         else:
             super().__call__(*args, **kwargs)
 
 
 class ForwardDriver(RouteDriver):
     """Alias to :class:`RouteDriver`"""
+
+
+class WhooshDriver(BaseControlDriver):
+    """Play a whoosh! sound"""
+
+    def __call__(self, *args, **kwargs):
+        """Play a whoosh sound, used in 2021 April fools day
+
+        .. # noqa: DAR101
+        """
+        import subprocess
+        from pkg_resources import resource_filename
+
+        whoosh_mp3 = resource_filename(
+            'jina', '/'.join(('resources', 'soundfx', 'whoosh.mp3'))
+        )
+
+        subprocess.Popen(
+            f'ffplay  -nodisp -autoexit {whoosh_mp3} >/dev/null 2>&1', shell=True
+        )
