@@ -30,18 +30,16 @@ class SliceQL(QuerySetReader, ContextAwareRecursiveMixin, BaseRecursiveDriver):
             traversal_paths: ['m']
 
     `SliceQL` will ensure that only the first 50 documents are returned from this `Pod`
+
+    :param start: Zero-based index at which to start extraction.
+    :param end:  Zero-based index before which to end extraction.
+            slice extracts up to but not including end. For example, take(1,4) extracts
+            the second element through the fourth element (elements indexed 1, 2, and 3).
+    :param args: additional positional arguments which are just used for the parent initialization
+    :param kwargs: additional key value arguments which are just used for the parent initialization
     """
 
     def __init__(self, start: int, end: Optional[int] = None, *args, **kwargs):
-        """
-
-        :param start: Zero-based index at which to start extraction.
-        :param end:  Zero-based index before which to end extraction.
-                slice extracts up to but not including end. For example, take(1,4) extracts
-                the second element through the fourth element (elements indexed 1, 2, and 3).
-        :param *args: *args for super
-        :param **kwargs: **kwargs for super
-        """
         super().__init__(*args, **kwargs)
         self._start = int(start)
         if end is None:
