@@ -5,14 +5,15 @@ __license__ = "Apache-2.0"
 
 import inspect
 from functools import wraps
-from typing import Callable, Any, Union, Iterator, List, Optional, Dict
+from itertools import islice, chain
+from typing import Callable, Any, Union, Iterator, List, Optional, Dict, Iterable
+import copy
 
 import numpy as np
 
 from .metas import get_default_metas
 from ..helper import batch_iterator, typename, convert_tuple_to_list
 from ..logging import default_logger
-from itertools import islice
 
 
 def as_aggregate_method(func: Callable) -> Callable:
@@ -70,7 +71,7 @@ def as_train_method(func: Callable) -> Callable:
 
 
 def wrap_func(cls, func_lst, wrapper):
-    """Wrapping a class method only once, inherited but not overrided method will not be wrapped again
+    """Wrapping a class method only once, inherited but not overridden method will not be wrapped again
 
     :param cls: class
     :param func_lst: function list to wrap
@@ -199,36 +200,32 @@ def _get_total_size(full_data_size, batch_size, num_batch):
     return total_size
 
 
-def _merge_results_after_batching(final_result, merge_over_axis: int = 0):
-    if len(final_result) == 1:
-        # the only result of one batch
-        return final_result[0]
+def _merge_results_after_batching(
+    final_result, merge_over_axis: int = 0, flatten: bool = True
+):
+    if not final_result:
+        return
 
-    if len(final_result) and merge_over_axis is not None:
-        if isinstance(final_result[0], np.ndarray):
+    if isinstance(final_result[0], np.ndarray):
+        if len(final_result[0].shape) > 1:
             final_result = np.concatenate(final_result, merge_over_axis)
-        elif isinstance(final_result[0], tuple):
-            reduced_result = []
-            num_cols = len(final_result[0])
-            for col in range(num_cols):
-                reduced_result.append(
-                    np.concatenate([row[col] for row in final_result], merge_over_axis)
-                )
-            final_result = tuple(reduced_result)
+    elif isinstance(final_result[0], list) and flatten:
+        final_result = list(chain.from_iterable(final_result))
 
-    if len(final_result):
-        return final_result
+    return final_result
 
 
 def batching(
-    func: Callable[[Any], np.ndarray] = None,
-    batch_size: Union[int, Callable] = None,
+    func: Optional[Callable[[Any], np.ndarray]] = None,
+    batch_size: Optional[Union[int, Callable]] = None,
     num_batch: Optional[int] = None,
     split_over_axis: int = 0,
     merge_over_axis: int = 0,
     slice_on: int = 1,
+    slice_nargs: int = 1,
     label_on: Optional[int] = None,
     ordinal_idx_arg: Optional[int] = None,
+    flatten_output: bool = True,
 ) -> Any:
     """Split the input of a function into small batches and call :func:`func` on each batch
     , collect the merged result and return. This is useful when the input is too big to fit into memory
@@ -240,10 +237,12 @@ def batching(
     :param merge_over_axis: merge over which axis into a single result
     :param slice_on: the location of the data. When using inside a class,
             ``slice_on`` should take ``self`` into consideration.
+    :param slice_nargs: the number of arguments
     :param label_on: the location of the labels. Useful for data with any kind of accompanying labels
     :param ordinal_idx_arg: the location of the ordinal indexes argument. Needed for classes
             where function decorated needs to know the ordinal indexes of the data in the batch
             (Not used when label_on is used)
+    :param flatten_output: If this is set to True, the results from different batches will be chained and the returning value is a list of the results. Otherwise, the returning value is a list of lists, in which each element is a list containing the result from one single batch. Note if there is only one batch returned, the returned result is always flatten.
     :return: the merged result as if run :func:`func` once on the input.
 
     Example:
@@ -266,12 +265,11 @@ def batching(
         def arg_wrapper(*args, **kwargs):
             # priority: decorator > class_attribute
             # by default data is in args[1] (self needs to be taken into account)
-            data = args[slice_on]
-            args = list(args)
-
+            data = args[slice_on : slice_on + slice_nargs]
             b_size = (
                 batch_size(data) if callable(batch_size) else batch_size
             ) or getattr(args[0], 'batch_size', None)
+
             # no batching if b_size is None
             if b_size is None or data is None:
                 return func(*args, **kwargs)
@@ -281,148 +279,61 @@ def batching(
                 f'num_batch={num_batch} axis={split_over_axis}'
             )
 
-            full_data_size = _get_size(data, split_over_axis)
-            total_size = _get_total_size(full_data_size, batch_size, num_batch)
-
-            final_result = []
-
+            results = []
             data = (data, args[label_on]) if label_on else data
 
-            yield_slice = isinstance(data, np.memmap)
+            yield_slice = [
+                isinstance(args[slice_on + i], np.memmap) for i in range(0, slice_nargs)
+            ]
+
             slice_idx = None
 
-            for b in batch_iterator(
-                data[:total_size], b_size, split_over_axis, yield_slice=yield_slice
-            ):
-                if yield_slice:
-                    slice_idx = b
-                    new_memmap = np.memmap(
-                        data.filename, dtype=data.dtype, mode='r', shape=data.shape
-                    )
-                    b = new_memmap[slice_idx]
-                    slice_idx = slice_idx[split_over_axis]
-                    if slice_idx.start is None or slice_idx.stop is None:
-                        slice_idx = None
-
-                if not isinstance(b, tuple):
-                    # for now, keeping ordered_idx is only supported if no labels
-                    args[slice_on] = b
-                    if ordinal_idx_arg and slice_idx is not None:
-                        args[ordinal_idx_arg] = slice_idx
-                else:
-                    args[slice_on] = b[0]
-                    args[label_on] = b[1]
-
-                r = func(*args, **kwargs)
-
-                if yield_slice:
-                    del new_memmap
-
-                if r is not None:
-                    final_result.append(r)
-
-            return _merge_results_after_batching(final_result, merge_over_axis)
-
-        return arg_wrapper
-
-    if func:
-        return _batching(func)
-    else:
-        return _batching
-
-
-def batching_multi_input(
-    func: Callable[[Any], np.ndarray] = None,
-    batch_size: Union[int, Callable] = None,
-    num_batch: Optional[int] = None,
-    split_over_axis: int = 0,
-    merge_over_axis: int = 0,
-    slice_on: int = 1,
-    num_data: int = 1,
-) -> Any:
-    """Split the input of a function into small batches and call :func:`func` on each batch
-    , collect the merged result and return. This is useful when the input is too big to fit into memory
-
-    :param func: function to decorate
-    :param batch_size: size of each batch
-    :param num_batch: number of batches to take, the rest will be ignored
-    :param split_over_axis: split over which axis into batches
-    :param merge_over_axis: merge over which axis into a single result
-    :param slice_on: the location of the data. When using inside a class,
-            ``slice_on`` should take ``self`` into consideration.
-    :param num_data: the number of data inside the arguments
-    :return: the merged result as if run :func:`func` once on the input.
-
-    ..warning:
-        data arguments will be taken starting from ``slice_on` to ``slice_on + num_data``
-
-    Example:
-        .. highlight:: python
-        .. code-block:: python
-
-            class MultiModalExecutor:
-
-                @batching_multi_input(batch_size = 64, num_data=2)
-                def encode(self, *batches, **kwargs):
-                    batch_modality0 = batches[0]
-                    embed0 = _encode_modality(batch_modality0)
-                    batch_modality1 = batches[1]
-                    embed1 = _encode_modality(batch_modality0)
-
-            class MemoryHungryRanker:
-
-                @batching_multi_input(batch_size = 64, slice_on = 2 , num_data=2)
-                def score(
-                    self, query_meta: Dict, old_match_scores: Dict, match_meta: Dict
-                ) -> 'np.ndarray':
-                ...
-    """
-
-    def _batching(func):
-        @wraps(func)
-        def arg_wrapper(*args, **kwargs):
-            data = args[slice_on]
-            # priority: decorator > class_attribute
-            # by default data is in args[1:] (self needs to be taken into account)
-            b_size = batch_size or getattr(args[0], 'batch_size', None)
-            # no batching if b_size is None
-            if b_size is None or data is None:
-                return func(*args, **kwargs)
-
-            args = list(args)
-            default_logger.debug(
-                f'batching enabled for {func.__qualname__} batch_size={b_size} '
-                f'num_batch={num_batch} axis={split_over_axis}'
-            )
-
-            # assume all datas have the same length
-            full_data_size = _get_size(args[slice_on], split_over_axis)
-            total_size = _get_total_size(full_data_size, b_size, num_batch)
-            final_result = []
-            yield_dict = [
-                isinstance(args[slice_on + i], Dict) for i in range(0, num_data)
-            ]
+            # split the data into batches
             data_iterators = [
                 batch_iterator(
-                    _get_slice(args[slice_on + i], total_size),
+                    data[i],
                     b_size,
                     split_over_axis,
-                    yield_dict=yield_dict[i],
+                    yield_slice=yield_slice[i],
                 )
-                for i in range(0, num_data)
+                for i in range(0, slice_nargs)
             ]
 
-            for batch in data_iterators[0]:
-                args[slice_on] = batch
-                for idx in range(1, num_data):
-                    args[slice_on + idx] = next(data_iterators[idx])
+            batch_args = list(copy.copy(args))
 
-                r = func(*args, **kwargs)
+            # load the batches of data and feed into the function
+            for _data_args in zip(*data_iterators):
+                _data_args = list(_data_args)
+                for i, (_yield_slice, _arg) in enumerate(zip(yield_slice, _data_args)):
+                    if _yield_slice:
+                        original_arg = args[slice_on + i]
+                        _memmap = np.memmap(
+                            original_arg.filename,
+                            dtype=original_arg.dtype,
+                            mode='r',
+                            shape=original_arg.shape,
+                        )
+                        _data_args[i] = _memmap[_arg]
+                        slice_idx = _arg[split_over_axis]
+                        if slice_idx.start is None or slice_idx.stop is None:
+                            slice_idx = None
+                        del _memmap
+
+                    # TODO: figure out what is ordinal_idx_arg
+                    if not isinstance(_data_args[i], tuple):
+                        if ordinal_idx_arg and slice_idx is not None:
+                            batch_args[ordinal_idx_arg] = slice_idx
+
+                batch_args[slice_on : slice_on + slice_nargs] = _data_args
+
+                r = func(*batch_args, **kwargs)
 
                 if r is not None:
-                    final_result.append(r)
+                    results.append(r)
 
-            return _merge_results_after_batching(final_result, merge_over_axis)
+            return _merge_results_after_batching(
+                results, merge_over_axis, flatten_output
+            )
 
         return arg_wrapper
 
@@ -433,47 +344,106 @@ def batching_multi_input(
 
 
 def single(
-    func: Callable[[Any], np.ndarray] = None,
+    func: Optional[Callable[[Any], np.ndarray]] = None,
     merge_over_axis: int = 0,
     slice_on: int = 1,
+    slice_nargs: int = 1,
+    flatten_output: bool = False,
 ) -> Any:
-    """
-    Guarantee that the input of a function is provided as a single instance and not in batches
+    """Guarantee that the inputs of a function with more than one argument is provided as single instances and not in batches
 
     :param func: function to decorate
     :param merge_over_axis: merge over which axis into a single result
     :param slice_on: the location of the data. When using inside a class,
             ``slice_on`` should take ``self`` into consideration.
+    :param slice_nargs: the number of positional arguments considered as data
+    :param flatten_output: If this is set to True, the results from different batches will be chained and the returning value is a list of the results. Otherwise, the returning value is a list of lists, in which each element is a list containing the result from one single batch. Note if there is only one batch returned, the returned result is always flatten.
     :return: the merged result as if run :func:`func` once on the input.
 
+    ..warning:
+        data arguments will be taken starting from ``slice_on` to ``slice_on + num_data``
+
     Example:
         .. highlight:: python
         .. code-block:: python
 
             class OneByOneCrafter:
+
                 @single
-                def craft(self, text: str) -> Dict:
+                def craft(self, text: str, id: str) -> Dict:
+            ...
+
+    .. note:
+        Single multi input decorator will let the user interact with the executor in 3 different ways:
+            - Providing batches: (This decorator will make sure that the actual method receives just a single instance)
+            - Providing a single instance
+            - Providing a single instance through kwargs.
+
+        .. highlight:: python
+        .. code-block:: python
+
+            class OneByOneCrafter:
+                @single
+                def craft(self, text: str, id: str) -> Dict:
+                    return {'text': f'{text}-crafted', 'id': f'{id}-crafted'}
+
+            crafter = OneByOneCrafter()
+
+            results = crafted.craft(['text1', 'text2'], ['id1', 'id2'])
+            assert len(results) == 2
+            assert results[0] == {'text': 'text1-crafted', 'id': 'id1-crafted'}
+            assert results[1] == {'text': 'text2-crafted', 'id': 'id2-crafted'}
+
+            result = crafter.craft('text', 'id')
+            assert result['text'] == 'text-crafted'
+            assert result['id'] == 'id-crafted'
+
+            results = crafted.craft(text='text', id='id')
+            assert result['text'] == 'text-crafted'
+            assert result['id'] == 'id-crafted'
     """
 
-    def _single(func):
+    def _single_multi_input(func):
         @wraps(func)
         def arg_wrapper(*args, **kwargs):
-            # priority: decorator > class_attribute
-            # by default data is in args[1] (self needs to be taken into account)
-            data = args[slice_on]
+            # by default data is in args[1:] (self needs to be taken into account)
             args = list(args)
-
             default_logger.debug(f'batching disabled for {func.__qualname__}')
 
+            data_iterators = args[slice_on : slice_on + slice_nargs]
+
+            if len(args) <= slice_on:
+                # like this one can use the function with single kwargs
+                return func(*args, **kwargs)
+            elif len(args) < slice_on + slice_nargs:
+                raise IndexError(
+                    f'can not select positional args at {slice_on}: {slice_nargs}, '
+                    f'your `args` has {len(args)} arguments.'
+                )
+            elif (
+                len(args) <= slice_on
+                or isinstance(data_iterators[0], str)
+                or isinstance(data_iterators[0], bytes)
+                or not isinstance(data_iterators[0], Iterable)
+            ):
+                # like this one can use the function with single kwargs
+                return func(*args, **kwargs)
+
             final_result = []
-            for instance in data:
-                args[slice_on] = instance
+            for new_args in zip(*data_iterators):
+                args[slice_on : slice_on + slice_nargs] = new_args
                 r = func(*args, **kwargs)
+
                 if r is not None:
                     final_result.append(r)
 
-            return _merge_results_after_batching(final_result, merge_over_axis)
+            return _merge_results_after_batching(
+                final_result, merge_over_axis, flatten=flatten_output
+            )
 
         return arg_wrapper
 
-    return _single(func)
+    if func:
+        return _single_multi_input(func)
+    else:
+        return _single_multi_input
