@@ -1,13 +1,13 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
-
 from typing import Optional
 
 import numpy as np
-from jina.excepts import UndefinedModel
-from jina.executors.decorators import batching
+
 from .. import BaseNumericEncoder
+from ...decorators import batching
+from ....excepts import UndefinedModel
 
 
 class TransformEncoder(BaseNumericEncoder):
@@ -15,12 +15,14 @@ class TransformEncoder(BaseNumericEncoder):
     :class:`TransformEncoder` encodes data from an ndarray in size `B x T` into an ndarray in size `B x D`
     """
 
-    def __init__(self,
-                 output_dim: int = 64,
-                 model_path: Optional[str] = None,
-                 random_state: int = 2020,
-                 *args,
-                 **kwargs):
+    def __init__(
+        self,
+        output_dim: int = 64,
+        model_path: Optional[str] = None,
+        random_state: int = 2020,
+        *args,
+        **kwargs
+    ):
         """
         :param model_path: path from where to pickle the sklearn model.
         """
@@ -30,7 +32,9 @@ class TransformEncoder(BaseNumericEncoder):
         self.random_state = random_state
 
     def post_init(self) -> None:
+        """Load the model from path if :param:`model_path` is set."""
         import pickle
+
         self.model = None
         if self.model_path:
             with open(self.model_path, 'rb') as model_file:
@@ -38,16 +42,21 @@ class TransformEncoder(BaseNumericEncoder):
 
     @batching
     def train(self, data: 'np.ndarray', *args, **kwargs) -> None:
+        """Train the :param:`data` with model."""
         if not self.model:
             raise UndefinedModel(
-                'Model is not defined: Provide a loadable pickled model, or defined any specific TransformEncoder')
+                'Model is not defined: Provide a loadable pickled model, or defined any specific TransformEncoder'
+            )
         num_samples, num_features = data.shape
         if not getattr(self, 'num_features', None):
             self.num_features = num_features
         if num_samples < 5 * num_features:
             self.logger.warning(
                 'the batch size (={}) is suggested to be 5 * num_features(={}) to provide a balance between '
-                'approximation accuracy and memory consumption.'.format(num_samples, num_features))
+                'approximation accuracy and memory consumption.'.format(
+                    num_samples, num_features
+                )
+            )
         self.model.fit(data)
         self.is_trained = True
 
