@@ -3,7 +3,7 @@ import re
 import tempfile
 import warnings
 from types import SimpleNamespace
-from typing import Dict, Any, Union, TextIO, Optional, List
+from typing import Dict, Any, Union, TextIO, Optional, List, Tuple
 
 import yaml
 from yaml.constructor import FullConstructor
@@ -108,16 +108,24 @@ class JAML:
         return re.sub(r, r'jtype: \1', value)
 
     @staticmethod
-    def unescape(value: str, include_unknown_tags: bool = True) -> str:
+    def unescape(
+        value: str,
+        include_unknown_tags: bool = True,
+        jtype_whitelist: Tuple[str, ...] = None,
+    ) -> str:
         """
         Unescape the YAML content by replacing all ``jtype: `` to tags.
 
         :param value: the escaped YAML content
         :param include_unknown_tags: if to include unknown tags during unescaping
+        :param jtype_whitelist: the list of jtype to be unescaped
         :return: unescaped YAML
         """
         if include_unknown_tags:
             r = r'jtype: (\w+)\b'
+        elif jtype_whitelist:
+            r = '|'.join(jtype_whitelist)
+            r = rf'jtype: ({r})\b'
         else:
             r = '|'.join(JAML.registered_tags())
             r = rf'jtype: ({r})\b'
@@ -142,7 +150,7 @@ class JAML:
         Load yaml object but ignore all customized tags, e.g. !Executor, !Driver, !Flow.
 
         :param stream: the output stream
-        :param **kwargs: other kwargs
+        :param kwargs: other kwargs
         :return: the Python object
         """
         safe_yml = JAML.escape('\n'.join(v for v in stream))
@@ -299,7 +307,7 @@ class JAML:
 
         :param data: the data to serialize
         :param stream: the output stream
-        :param **kwargs: other kwargs
+        :param kwargs: other kwargs
         :return: the yaml output
         """
         return yaml.dump(
@@ -484,7 +492,7 @@ class JAMLCompatible(metaclass=JAMLCompatibleType):
         :param allow_py_modules: allow importing plugins specified by ``py_modules`` in YAML at any levels
         :param substitute: substitute environment, internal reference and context variables.
         :param context: context replacement variables in a dict, the value of the dict is the replacement.
-        :param **kwargs: **kwargs for parse_config_source
+        :param kwargs: kwargs for parse_config_source
         :return: :class:`JAMLCompatible` object
         """
         stream, s_path = parse_config_source(source, **kwargs)
@@ -507,8 +515,17 @@ class JAMLCompatible(metaclass=JAMLCompatibleType):
                     no_tag_yml,
                     extra_search_paths=(os.path.dirname(s_path),) if s_path else None,
                 )
-            # revert yaml's tag and load again, this time with substitution
-            tag_yml = JAML.unescape(JAML.dump(no_tag_yml))
+            from ..flow import BaseFlow
+
+            if issubclass(cls, BaseFlow):
+                tag_yml = JAML.unescape(
+                    JAML.dump(no_tag_yml),
+                    include_unknown_tags=False,
+                    jtype_whitelist=('Flow', 'AsyncFlow'),
+                )
+            else:
+                # revert yaml's tag and load again, this time with substitution
+                tag_yml = JAML.unescape(JAML.dump(no_tag_yml))
 
             # load into object, no more substitute
             return JAML.load(tag_yml, substitute=False)
@@ -521,8 +538,8 @@ class JAMLCompatible(metaclass=JAMLCompatibleType):
             This function is most likely to be overridden by its subclass.
 
         :param raw_config: raw config to work on
-        :param *args: *args
-        :param **kwargs: **kwargs
+        :param args: not used
+        :param kwargs: not used
         :return: the config
         """
         return raw_config
